#!/usr/bin/ruby -w

# Scotty TNG - ping/snmp based realtime network monitoring
#
# $Id: daemon.rb 140 2008-10-08 21:04:31Z liske $
#
# Authors:
#   Thomas Liske <liske@ibh.de>
#   Andre Ellguth <ellguth@ibh.de>
#
# Copyright Holder:
#   2008 (C) IBH IT-Service GmbH [http://www.ibh.de/]
#
# License:
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this package; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
#

pwb = Dir.pwd
begin
    Dir.chdir('/usr/lib/scotty-tng/daemon')
rescue
end

require 'rexml/document'
require 'drbd.rb'
require 'node.rb'
require 'stack.rb'
require 'sensor.rb'
require 'sensors/if.rb'
require 'sensors/rtt.rb'
require 'sensors/ups.rb'
require 'ringlist.rb'

begin
    Dir.chdir(pwd)
rescue
end

begin
  file = "config/config.xml"

  puts "Loading file '" + file + "'"
  xml = REXML::Document.new(File.open(file))
  puts

  puts "Loading nodes..."
  xml.elements.each("/config/nodes/node") { |xmlnode|
    Node.addNode(xmlnode)
  }
  puts

  puts "Loading stacks..."
  xml.elements.each("/config/nodes/stack") { |xmlstack|
    Stack.addStack(xmlstack)
  }
  puts
end

puts "Starting dRuby..."
ScottyTNG.new(xml.elements["/config/drb"].attributes["uri"])
puts

puts "\nStart monitoring...\n"

threads = []
threads << Thread.new {
    Sensors::RTT::monitor
}
threads << Thread.new {
    Sensors::IF::monitor
}
threads << Thread.new {
    Sensors::UPS::monitor
}

puts "\nRunning...\n"

threads.each{ |t| t.join }

puts "\nTerminated.\n"
