# Scotty TNG - ping/snmp based realtime network monitoring
#
# $Id: dringld.rb 95 2008-09-30 10:38:37Z liske $
#
# Authors:
#   Thomas Liske <liske@ibh.de>
#   Andre Ellguth <ellguth@ibh.de>
#
# Copyright Holder:
#   2008 (C) IBH IT-Service GmbH [http://www.ibh.de/]
#
# License:
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this package; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
#

# This is a implementation of a distributed ringlist in a dRuby tuplespace.
# This is no generic ringlist, actually it isn't a ringlist at all. The tuplespace
# will hold only one value tuple, the ringlist is recorded on the clients observing
# value tuple writes.
class DRingLD
  def initialize(ts, id)
    @ts, @id = ts, id
  end

  def self.create(ts, name, options)
    result = new(ts, name)
    result.init(options)
    result
  end

  class << self
    alias_method :find, :new
  end

  def init(options)
    @ts.write [:DRingLVals, @id[0], @id[1], nil]
    @ts.write [:DRingLOpts, @id[0], @id[1], options]
  end

  def push(value)
    t = @ts.take [:DRingLVals, @id[0], @id[1], nil]
    t[3] = value
    @ts.write t
  end
end
