# Scotty TNG - ping/snmp based realtime network monitoring
#
# $Id: sensor.rb 141 2008-10-09 06:34:10Z liske $
#
# Authors:
#   Thomas Liske <liske@ibh.de>
#   Andre Ellguth <ellguth@ibh.de>
#
# Copyright Holder:
#   2008 (C) IBH IT-Service GmbH [http://www.ibh.de/]
#
# License:
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this package; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
#

# Basic abstract sensor class.
class Sensor
  @@_sens = Hash.new

  attr_reader :node, :thresholds, :name, :monitor, :stacks

  def add_expr(name, &block)
    @expr[name] = block
  end

  def initialize(node, xmlsensor, name)
    @node = node
    @name = name
    @@_sens["#{@node.name}/#{@name}"] = self
    @monitor = Array.new
    @expr = Hash.new
    @stacks = Array.new
    @status = ["ok", ""]
    @class = xmlsensor.attributes["class"]

    # Initial status is "ok"
    self.set_status("ok", nil, true)

    # Read thresholds...
    begin
	xmlsensor.elements["thresholds"].elements.each {|mon|
	    @monitor.push([
		      mon.attributes["expr"],
		      mon.attributes["status"],
		      mon.attributes["msg"],
		    ])
	}
    rescue
	begin
	    # Read defaults...
	    xmlsensor.root.elements["/config/defaults/" + xmlsensor.name + "/thresholds"].elements.each {|mon|
		@monitor.push([
		      mon.attributes["expr"],
		      mon.attributes["status"],
		      mon.attributes["msg"],
		    ])
	    }
	rescue
	    # No thresholds defined for this sensor.
	end
    end
  end

  def set_status(status, msg, startup=false)
    if @status[0] != status or @status[1] != msg then
      # Consume any pending status from TS
      begin
	ScottyTNG.getTS.take([:SensorStatus, @node.name, @name, nil], 0) while true
      rescue
      end

      # Write new status to TS
      ScottyTNG.getTS.write [:SensorStatus, @node.name, @name, [Time.now.to_i, status, msg]]
      @status = [status, msg]

      puts "#{@node.name}/#{@name}: status '#{status}': #{msg}" unless startup
    end
  end

  def do_check(m)
    begin
      b = binding().clone

      @expr.each_pair{|k, v|
	eval("#{k} = #{v.call}", b)
      }

      eval(m, b)
    rescue
      puts "Error eval monitor expression: #{$!}"
    end
  end

  def check_mon
    @monitor.each{|m|
      if do_check(m[0]) then
	self.set_status(m[1], m[2])
	return
      end
    }
    self.set_status("ok", nil)
  end

  def addStack(stack)
    @stacks.push(stack)
  end

  def pushProbe(value)
    @node.ringlists[@name].pushProbe(value)
    @stacks.each{|s|
	s.pushProbe("#{@node.name}/#{@name}", value)
    }
  end

  def self.create(node, xmlsensor)
    case xmlsensor.name
    when "rtt"
      return Sensors::RTT.new(node, xmlsensor)
    when "if"
      return Sensors::IF.new(node, xmlsensor)
    when "ups"
      return Sensors::UPS.new(node, xmlsensor)
    else
      puts "Could not find sensor class for:\n" + xmlsensor.to_s + "\n"
    end
  end
  
  def self.getSensor(node, name)
    @@_sens["#{node}/#{name}"]
  end
end
