#!/usr/bin/perl -w

# Scotty TNG - ping/snmp based realtime network monitoring
#
# $Id: tki2config.pl 69 2008-09-21 17:51:23Z liske $
#
# Authors:
#   Thomas Liske <liske@ibh.de>
#   Andre Ellguth <ellguth@ibh.de>
#
# Copyright Holder:
#   2008 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
#
# License:
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this package; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
#
# This is a hack for converting *.tki files of the original scotty program. It
# might work, but don't use it seriously.
#

use strict;

my $in = 0;
my @block;

my %nodes;
my %stripcharts;
my %ifs;

while(<>) {
    chomp;

    if ($_ eq '') {
	if ($in == 1) {
	    my $b = join('|', @block);

	    $b =~ /noupdate create (\S+) /;
	    my $type = $1;
	
	    if ($type eq 'NODE') {
		$b =~ /name \$\S+ (\S+|\{([^|]+)\})\|/;
		my $name = $1;
		$name = $2 if $2;
		$b =~ /address \$\S+ (\S+)\|/;
		my $address = $1;

		$address = &hostname($address) unless($address =~ /^[\d.]+$/);

		${$nodes{lc($name)}}{'name'} = $name;
		${$nodes{lc($name)}}{'address'} = $address;
	    }
	    elsif ($type eq 'STRIPCHART' && $b =~ /round trip/) {
		$b =~ /name \$(\S+) (\S+|\{([^|]+)\})\|/;
		my $name = $2;
		$name = $3 if $3;
		my $strip = $1;
		$b =~ /address \$\S+ (\S+)\|/;
		my $address = $1;
		$address = &hostname($address) unless($address =~ /^[\d.]+$/);

		print STDERR "WARNING: STRIPCHART $strip has no address assigned!\n" if ($name eq '{}');

		${$nodes{lc($name)}}{'name'} = $name;
		${$nodes{lc($name)}}{'address'} = $address;
	    }
	    elsif ($type eq 'INTERPRETER' && $b =~ /snmp_monitor\.tcl/) {
		while($b =~ s/ined send [^|]+ restart start_ifload_monitor \d+ job\d+ ([\d.]+) ([^|]+)\|//) {
		    my ($nodeip, $charts) = ($1, $2);
		    while($charts =~ s/\$strip[\w\d]+ (\d+)//) {
			push(@{$ifs{$nodeip}}, $1);
		    }
		}
	    }
	}

	$in = 1;
	@block = ();
	next;
    }

    push(@block, $_) if ($in == 1 && $_ ne '');
}

foreach my $name (keys %nodes) {
    print "	    <node name='", ${$nodes{$name}}{'name'},"' address='", ${$nodes{$name}}{'address'}, "'>
		<rtt/>\n";
    foreach my $idx (@{$ifs{${$nodes{$name}}{'address'}}}) {
	print "		<if idx='", $idx, "'/>\n";
    }
    delete($ifs{${$nodes{$name}}{'address'}});
    print "	    </node>\n";
}

print "Unassigned stripcharts: ", join(", ", keys %ifs), "\n";

sub hostname {
  my (@bytes, @octets,
    $packedaddr,
    $raw_addr,
    $host_name,
    $ip
  );

  if($_[0] =~ /[a-zA-Z]/g) {
    $raw_addr = (gethostbyname($_[0]))[4];
    @octets = unpack("C4", $raw_addr);
    $host_name = join(".", @octets);
  } else {
    @bytes = split(/\./, $_[0]);
    $packedaddr = pack("C4",@bytes);
    $host_name = (gethostbyaddr($packedaddr, 2))[0];
  }

  return($host_name);
}
